#encoding:utf-8
import datetime

from lib.protocol.Base import Base

'''
定义消息协议类的基类
'''

class MessageBase(Base):

    def __init__(self):
        self.IDENTIFY = "7e"     #标识位

    #######################################################
    # 生成一条完整的消息
    #######################################################
    def generateMsg(self):
        msg = ""
        msgHeader = self.getMsgHeader()
        msgBody = self.getMsgBody()
        checkCode = self.getCheckCode(msgHeader + msgBody)
        msg = msg + self.IDENTIFY
        info = msgHeader + msgBody + checkCode
        info = self.replace7e7d(info)
        msg = msg + info
        msg = msg + self.IDENTIFY
        return msg


    #######################################################
    # 获取消息头
    #######################################################
    def getMsgHeader(self):
        msgID = self.int2hexStringByBytes(102,2)                 #消息id
        subPkg = 0
        msgBodyProperty = self.getMsgBodyProperty(subPkg=subPkg)              #消息体属性
        phoneNum = self.int2BCD(13146201118)                     #终端手机号
        msgWaterCode = self.int2hexStringByBytes(1)              #消息流水号
        if subPkg != 8192:
            subPkgContent = ""                                       #消息包封装项
        else:
            subPkgContent = self.getMsgPackage()
        data = msgID + msgBodyProperty + phoneNum + msgWaterCode + subPkgContent
        return data

    #获取消息体属性
    def getMsgBodyProperty(self,msgBodyLen=128,encryptionType=0,subPkg=0):
        if msgBodyLen >= 512:
            raise RuntimeError('消息体长度超长！')
        msgBodyLen = msgBodyLen                                  #消息体长度
        encryptionType = encryptionType                          #加密方式
        subPkg = subPkg                                          #分包
        retain = 0                                               #保留位
        data = msgBodyLen + encryptionType + subPkg + retain
        dataHex = self.int2hexStringByBytes(data,2)
        return dataHex

    #获取消息封装项
    def getMsgPackage(self):
        pkgCounts = 2                   #消息报包总数
        pkgCountsHex = self.int2hexStringByBytes(2,2)
        pkgNumsHex = ""
        for i in range(0,pkgCounts):
            pkgNum = i
            pkgNumHex = self.int2hexStringByBytes(pkgNum,2)
            pkgNumsHex = pkgNumsHex + pkgNumHex
        msgPackage = pkgCountsHex + pkgNumsHex
        return msgPackage



    #######################################################
    # 获取消息体
    #######################################################
    def getMsgBody(self):
        return ""

    #######################################################
    # 获取校验码
    #######################################################
    def getCheckCode(self,data="aa"):
        if len(data) % 2 == 1:
            raise RuntimeError('数据段错误！')
        start = data[0:2]
        tmp = int(start,16)
        for i in range(2,len(data),2):
            tmp = tmp ^ int(data[i:i + 2],16)
        dataHex = self.int2hexStringByBytes(tmp)
        return dataHex

    #######################################################
    # 替换消息中的7e7d字符
    #######################################################
    def replace7e7d(self,data):
        data = data.replace("7d","7d01")
        data = data.replace("7e","7d02")
        return data

    #######################################################
    # 字符串转16进制
    #######################################################
    def str2Hex(self,data):
        dataHex = ""
        tem = ""
        for i in data:
            tem = ord(i)
            dataHex += hex(tem)[2:]
        return dataHex

    #####################################################
    # 将字符串转换为对应的ascii值数组
    #####################################################
    def str2Ascsii(self,data):
        asciiArr = []
        for i in range(0, len(data)):
            asciiValue = ord(data[i])
            asciiArr.append(asciiValue)
        return asciiArr

    ####################################################
    # 将整数转换为有符号的整数
    #####################################################
    def num2signedNum(self,num):
        return num & 0xff

    #####################################################
    # 数字转换为16进制字符串，通过传入字节数可自动补0
    # 传入数据格式所占字节数
    #####################################################
    def int2hexStringByBytes(self, num,bytescount=1):
        hexStr = hex(num)[2:]
        while len(hexStr) < (bytescount * 2):
            hexStr = "0" + hexStr
        return hexStr

    ####################################################
    # 将10进制转换位8421码
    #####################################################
    def int2BCD(self,data):
        data = str(data)
        while len(data) < 12:
            data = "0" + data
        return data

    #######################################################
    # 获取UTC时间转换位BCD格式
    #######################################################
    def getBCDTime(self,data="2020-02-04 18:57:04"):
        now_time = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
        data = data
        data = data.replace("-","")
        data = data.replace(" ","")
        data = data.replace(":","")
        data = data[2:]
        data = self.int2BCD(int(data))
        return data



if __name__ == "__main__":
    # print(MessageBase().str2Hex("uvwxyz"))
    # print(MessageBase().str2Ascsii("uvwxyz"))
    print(MessageBase().int2hexStringByBytes(220400566542345564784802,20))
    print(MessageBase().str2Hex("a865h643gfdj64fd7432"))
    print(MessageBase().str2Hex("\xd3\xe5B23CX"))
    # print(MessageBase().getMsgBodyProperty())
    # print(MessageBase().int2BCD(13146201117))
    # print(MessageBase().getCheckCode("8001000501314620111800000000000200"))
    # print(MessageBase().getMsgHeader())
    # print(MessageBase().generateMsg())