#encoding:utf-8

'''
定义轿车OBD数据
'''
from lib.protocol.message.MessageBase import MessageBase


class SaloonCarOBD_data(MessageBase):
    def __init__(self):
        super().__init__()
        pass

    #####################################################
    # 创建轿车OBD数据
    #####################################################
    def generateSaloonCarOBDData(self):
        data = ""
        #发动机转速 , 0 - 8000  rpm
        engineSpeed = "60C0" + self.int2hexStringByBytes(2) + self.int2hexStringByBytes(3000,2)
        #车速 ， 0 - 240 Km/h
        carSpeed = "60D0" + self.int2hexStringByBytes(1) + self.int2hexStringByBytes(60)
        #剩余油量 ，剩余油量，单位 L 或%Bit15=0 百分比%，OBD 都为百分比Bit15=1 单位 L，显示值为上传值/10
        surplusOil = "62f0" + self.int2hexStringByBytes(2) + self.getSurplusOil()
        #冷却液温度 ，-40.0℃ 到 +210℃,上传值减去 40
        coolingLiquidTemperature = "6050" + self.int2hexStringByBytes(1) + self.int2hexStringByBytes(76)
        #进气口温度 ，-40.0℃ 到 +210℃,上传值减去 40
        airInletTemperature = "60F0" + self.int2hexStringByBytes(1) + self.int2hexStringByBytes(88)
        #进气(岐管绝对)压力 , 0 - 250kpa
        intakeManifoldPressure = "60B0" + self.int2hexStringByBytes(1) + self.int2hexStringByBytes(20)
        #大气压力 , 0 - 125kpa
        atmosphericPressure = "6330" + self.int2hexStringByBytes(1) + self.int2hexStringByBytes(28)
        #环境温度 ， -40.0℃ 到 +210℃,上传值减去 40
        envTemperature = "6460" + self.int2hexStringByBytes(1) + self.int2hexStringByBytes(65)
        #加速踏板位置 ， 0% - 100%
        acceleratorLocation = "6490" + self.int2hexStringByBytes(1) + self.int2hexStringByBytes(32)
        #燃油压力 ， 0 - 500kpa
        oilPressure = "60A0" + self.int2hexStringByBytes(2) + self.int2hexStringByBytes(276,2)
        #故障码状态 ， 发动机故障码状态
        troubleCodeStatus = "6014" + self.int2hexStringByBytes(1) + self.int2hexStringByBytes(0)
        #故障码个数
        troubleCodeNum = "6010" + self.int2hexStringByBytes(1) + self.int2hexStringByBytes(2)
        #空气流量 ， 0.1	实际值为上传值/10 0.1g/s
        airFlow = "6100" + self.int2hexStringByBytes(2) + self.int2hexStringByBytes(550,2)
        #绝对气门位置
        valveLocation = "6110" + self.int2hexStringByBytes(2) + self.int2hexStringByBytes(501,2)
        #自发动机启动运行时间 sec
        engineRunTime = "61F0" + self.int2hexStringByBytes(2) + self.int2hexStringByBytes(3700,2)
        #故障行驶里程 ， Km
        troubleMileage = "6210" + self.int2hexStringByBytes(4) + self.int2hexStringByBytes(4508,4)
        #计算负荷值
        calculateLoadValue = "6040" + self.int2hexStringByBytes(1) + self.int2hexStringByBytes(44)
        #长期燃油修正(气缸列 1 和 3)
        fuelTrim = "6070" + self.int2hexStringByBytes(2) + self.int2hexStringByBytes(56,2)
        #第一缸点火正时提前角 ,显示值为上传值-64
        fireAngle = "60E0" + self.int2hexStringByBytes(2) + self.int2hexStringByBytes(80)
        #前刹车片磨损 , 0 正常/否则，显示对应数据，单位：级
        frontBrakeBlockAbrasion = "6701" + self.int2hexStringByBytes(1) + self.int2hexStringByBytes(0)
        #后刹车片磨损 , 0 正常/否则，显示对应数据，单位：级
        backBrakeBlockAbrasion = "6702" + self.int2hexStringByBytes(1) + self.int2hexStringByBytes(0)
        #制动液液位 , 0：不正 1：正常 其他：无法使用
        brakeFluidLocation = "6703" + self.int2hexStringByBytes(1) + self.int2hexStringByBytes(1)
        #机油液位 , 显示值为上传值/1000 单位 毫米
        engineOilLocation = "6704" + self.int2hexStringByBytes(2) + self.int2hexStringByBytes(20000,2)
        #胎压报警 0：当前无警告   1：存在胎压失压 其他：不可用
        tirePressureAlarm = "6705" + self.int2hexStringByBytes(2) + self.int2hexStringByBytes(0,2)
        #冷却液液位 , 显示值为上传值-48
        coolingLiquidLocation = "6706" + self.int2hexStringByBytes(2) + self.int2hexStringByBytes(65,2)
        #续航里程 0.1 km ; 显示值为上传值/10
        enduranceMileage = "6707" + self.int2hexStringByBytes(4) + self.int2hexStringByBytes(380,4)

        data = data + engineSpeed + carSpeed + surplusOil + coolingLiquidTemperature + airInletTemperature
        data = data + intakeManifoldPressure + atmosphericPressure + envTemperature + acceleratorLocation + oilPressure
        data = data + troubleCodeStatus + troubleCodeNum + airFlow + valveLocation + engineRunTime
        data = data + troubleMileage + calculateLoadValue + fuelTrim + fireAngle + frontBrakeBlockAbrasion
        data = data + backBrakeBlockAbrasion + brakeFluidLocation + engineOilLocation + tirePressureAlarm + coolingLiquidLocation
        data = data + enduranceMileage
        return data

    #####################################################
    # 获取剩余油量
    #####################################################
    def getSurplusOil(self):
        # 剩余油量 ，剩余油量，单位 L 或%Bit15=0 百分比%，OBD 都为百分比Bit15=1 单位 L，显示值为上传值/10   (32768)
        units = 32768
        surplusOil = 800
        data = units + surplusOil
        dataHex = self.int2hexStringByBytes(data,2)
        return dataHex


if __name__ == "__main__":
    print(SaloonCarOBD_data().generateSaloonCarOBDData())