#coding: utf-8
import logging
import threading
from time import sleep

from lib.socket.SocketBase import SocketBase
from lib.socket.websocket_server import WebsocketServer

class MyWebsocket_server(SocketBase):
    def __init__(self,host="127.0.0.1",port=5005):
        self.connectTimeout = 5       #设置socket服务启动之后的连接超时时间
        self.connectTimeoutstatus = 0      #设置socket服务启动之后是否有连接，0为为收到连接，1:为已经收到了连接
        self.msgTimeout = 12               #设置12秒没有收到任何消息之后，就关闭连接
        self.host = host
        self.port = port


    def startWebsocketServer(self):
        server = WebsocketServer(self.port, host=self.host, loglevel=logging.INFO)
        server.set_fn_new_client(self.new_client)
        server.set_fn_message_received(self.mysend)
        self.connectTimeoutThreading(server)    #启动超时连接线程
        self.msgTimeoutThreading(server)        #启动接收消息超时连接线程
        server.run_forever()
        server.server_close()

    #有客户端连接成功之后，回复一条消息
    def new_client(self,client, server):
        server.send_message_to_all("连接成功......")
        self.connectTimeout = 1

    #收到消息之后的恢复消息方法
    def mysend(self,client,server,msg):
        print(msg)
        self.msgTimeout = 12
        server.send_message(client,"哈哈哈哈哈")
        if(msg == "_end"):      #收到_end 控制码的时候，断开连接
            server.server_close()
            print("连接断开...")

    ###################################################
    #         超过时间没有客户端连接，自动断开连接线程
    ###################################################
    def connectTimeoutThreading(self,server):
        t1 = threading.Thread(target=self.doConnectTomeout, args=(server,))
        t1.start()


    ###################################################
    #         超过时间没有客户端连接，自动断开连接方法
    ###################################################
    def doConnectTomeout(self,server):
        while self.connectTimeout > 0:
            self.connectTimeout -= 1
            sleep(1)
        if self.connectTimeoutstatus == 0:
            server.server_close()
            print("连接断开...")
        else:
            pass

    ###################################################
    #         超过时间没有收到任何消息，自动断开连接线程
    ###################################################
    def msgTimeoutThreading(self, server):
        t1 = threading.Thread(target=self.doMsgTimeout, args=(server,))
        t1.start()


    ###################################################
    #         超过时间没有收到任何消息，自动断开连接方法
    ###################################################
    def doMsgTimeout(self, server):
        while self.msgTimeout > 0:
            self.msgTimeout -= 1
            sleep(1)
        if self.connectTimeout == 1:
            server.server_close()
            print("连接断开...")















